/*
 * button.cc
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <Arduino.h>
#include <button.h>


Button::Button( int pin )
	:
	_pin( pin ),
	_state( false ),
	_millis_state( 0 ),
	_millis_done( 0 ),
	_pending_event( 0 ),
	_last_real_state( false ),
	_millis_real_state( 0 )
{
}


void Button::set_event_times( int event_times[] )
{
	_event_times = event_times;
}


void Button::update()
{
	// get time
	unsigned long millis = ::millis();


	// get button state
	bool real_state = digitalRead( _pin )? false : true;

	// if real state has changed, reset the timer
	if( real_state != _last_real_state ) {
		_last_real_state = real_state;
		_millis_real_state = millis;
	}

	// work out current state; if the real state timer has elapsed, set our
	// state to real state
	bool state = _state;
	if( _last_real_state != state && millis - _millis_real_state > 15 )
		state = _last_real_state;

	// if the button is pressed
	if( state )
	{
		// if it has just been pressed, record the time now and reset how much
		// of the press we have already generated events for
		if( !_state ) {
			_millis_state = millis;
			_millis_done = 0;
		}

		// calculate time that has elapsed in total this press
		unsigned long elapsed = millis - _millis_state;

		// look through the events to see if any need to be triggered
		for( int a = 0; _event_times[ a ]; a++ )
		{
			// if this event is in the future, stop
			if( (unsigned)_event_times[ a ] > elapsed ) break;

			// if this event has already been triggered, skip
			if( (unsigned)_event_times[ a ] <= _millis_done ) continue;

			// trigger event
			_pending_event = a + 1;
		}

		// update the amount of press we have processed
		_millis_done = elapsed;
	}

	// update our state
	_state = state;
}


int Button::get_event()
{
	int event = _pending_event;
	_pending_event = 0;
	return event;
}


bool Button::get_state()
{
	return _state;
}
